package com.sulake.habbo;

import java.math.BigInteger;

import com.sulake.habbo.catalog.CatalogManager;
import com.sulake.habbo.communication.ServerManager;
import com.sulake.habbo.communication.messages.MessageHandler;
import com.sulake.habbo.communication.remote.RemoteServerManager;
import com.sulake.habbo.configuration.Config;
import com.sulake.habbo.currencies.CurrencyManager;
import com.sulake.habbo.database.DatabaseManager;
import com.sulake.habbo.flats.FlatManager;
import com.sulake.habbo.flats.categories.CategoryManager;
import com.sulake.habbo.flats.items.definitions.ItemDefinitionManager;
import com.sulake.habbo.flats.items.interactions.InteractionManager;
import com.sulake.habbo.flats.models.ModelManager;
import com.sulake.habbo.game.encryption.DiffieHellman;
import com.sulake.habbo.game.encryption.HabboCrypto;
import com.sulake.habbo.settings.ServerSettings;
import com.sulake.habbo.users.ranks.RankManager;
import com.sulake.habbo.utilities.Logger;
import com.sulake.sessions.SessionManager;

public class Core {

	private ServerManager serverManager;
	private SessionManager sessionManager;
	private MessageHandler messageHandler;
	public DatabaseManager databaseManager;
	private ServerSettings serverSettings;
	private Config configHandler;
	private ModelManager modelManager;
	private FlatManager flatManager;
	private CategoryManager categoryManager;
	private ItemDefinitionManager itemManager;
	private RankManager rankManager;
	private CatalogManager catalogManager;
	private RemoteServerManager remoteServerManager;
	private InteractionManager interactionManager;
	
	// Unneeded cryptoshit
	public static HabboCrypto globalCrypto;
    public static String publicToken;
    private static BigInteger n = new BigInteger("86851DD364D5C5CECE3C883171CC6DDC5760779B992482BD1E20DD296888DF91B33B936A7B93F06D29E8870F703A216257DEC7C81DE0058FEA4CC5116F75E6EFC4E9113513E45357DC3FD43D4EFAB5963EF178B78BD61E81A14C603B24C8BCCE0A12230B320045498EDC29282FF0603BC7B7DAE8FC1B05B52B2F301A9DC783B7", 16);
    private static BigInteger e = new BigInteger("3");
    private static BigInteger d = new BigInteger("59AE13E243392E89DED305764BDD9E92E4EAFA67BB6DAC7E1415E8C645B0950BCCD26246FD0D4AF37145AF5FA026C0EC3A94853013EAAE5FF1888360F4F9449EE023762EC195DFF3F30CA0B08B8C947E3859877B5D7DCED5C8715C58B53740B84E11FBC71349A27C31745FCEFEEEA57CFF291099205E230E0C7C27E8E1C0512B", 16);
	
	public Core() {
		
		publicToken = new BigInteger(DiffieHellman.GenerateRandomHexString(15), 16).toString();
        globalCrypto = new HabboCrypto(n, e, d);
        
		this.configHandler = new Config("/server.properties");
		
		String ip = this.getConfig().getProperties().getProperty("game.ip");
		int port = Integer.parseInt(this.getConfig().getProperties().getProperty("game.port"));
		int web_port = Integer.parseInt(this.getConfig().getProperties().getProperty("web.port"));
			
		this.databaseManager = new DatabaseManager("jdbc:mysql://" +  this.getConfig().getProperties().getProperty("database.host") +"/" + this.getConfig().getProperties().getProperty("database.name"), this.getConfig().getProperties().getProperty("database.user"), this.getConfig().getProperties().getProperty("database.password"));
		Logger.write("MySQL database connection has been made.");
	
		this.sessionManager = new SessionManager();
		this.interactionManager = new InteractionManager();
		this.modelManager = new ModelManager();
		this.flatManager = new FlatManager(this.databaseManager);
		this.categoryManager = new CategoryManager();
		this.itemManager = new ItemDefinitionManager();
		this.rankManager = new RankManager();
		this.catalogManager = new CatalogManager();
		
		this.serverSettings = new ServerSettings(this.databaseManager);
		
		this.messageHandler = new MessageHandler();
		this.loadCache();
		
		ServerSettings.get("bannerdata").value = Core.globalCrypto.dHell.Prime + ":" + Core.globalCrypto.dHell.Generator;
		this.getServerSettings().save(ServerSettings.get("bannerdata"));
		
		this.remoteServerManager = new RemoteServerManager(ip, web_port);
		Logger.write("Remote Server has successfully booted up, on the server port: " + web_port + "! (can be accessed via http://" + ip + ":" + web_port + "/)");
		
		this.serverManager = new ServerManager(ip, port);
		Logger.write("Server has successfully booted up, on the server port: " + port + "!");
		Logger.writeSeperator();	
	}
	
	public void loadCache() {
		this.getMessageHandler().addHandledMessages();
		this.getServerSettings().loadAll();
		this.getModels().loadAll(this.getDatabase());
		this.getCategories().loadAll(this.getDatabase());
		this.getItems().loadAll(this.getDatabase());
		this.getRanks().loadAll(this.getDatabase());
		this.getCatalog().loadAll(this.getDatabase());

		CurrencyManager.loadAll(this.getDatabase());
	}
	
	public CatalogManager getCatalog() {
		return this.catalogManager;
	}

	public RankManager getRanks() {
		return this.rankManager;
	}

	public static String clean(String str) {
		str = str.replace((char) 1, ' ');
		str = str.replace((char) 2, ' ');
		str = str.replace((char) 3, ' ');
		str = str.replace((char) 9, ' ');
		
		return str;
	}
	
	public ServerSettings getServerSettings() {
		return this.serverSettings;
	}
	
	public MessageHandler getMessageHandler() {
		return this.messageHandler;
	}
	public SessionManager getSessions() {
		return this.sessionManager;
	}
	
	public ServerManager getServer() {
		return this.serverManager;
	}
	
	public RemoteServerManager getRemoteServer() {
		return this.remoteServerManager;
	}
	
	public DatabaseManager getDatabase() {
		return this.databaseManager;
	}
	
	public ModelManager getModels() {
		return this.modelManager;
	}
	
	public FlatManager getFlats() {
		return this.flatManager;
	}
	
	public CategoryManager getCategories() {
		return this.categoryManager;
	}
	
	public ItemDefinitionManager getItems() {
		return this.itemManager;
	}
	
	public InteractionManager getInteractions() {
		return this.interactionManager;
	}
	
	public Config getConfig() {
		return this.configHandler;
	}
	
}
