package com.sulake.habbo.communication;

import java.net.InetSocketAddress;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.jboss.netty.bootstrap.ServerBootstrap;
import org.jboss.netty.channel.ChannelPipeline;
import org.jboss.netty.channel.socket.nio.NioServerSocketChannelFactory;
import org.jboss.netty.handler.execution.ExecutionHandler;
import org.jboss.netty.handler.execution.OrderedMemoryAwareThreadPoolExecutor;

import com.sulake.habbo.communication.codec.Decoder;
import com.sulake.habbo.communication.codec.Encoder;
import com.sulake.habbo.communication.messages.MessageHandler;
import com.sulake.sessions.SessionManager;

public class ServerManager {

	private NioServerSocketChannelFactory channelFactory;
	private ServerBootstrap serverBootstrap;

	public SessionManager sessionManager;
	
	public MessageHandler messageHandler;
	
	public ServerManager(String ip, int port) {
		
		this.channelFactory = new NioServerSocketChannelFactory(
				Executors.newCachedThreadPool(),
				Executors.newCachedThreadPool()
		);
		
		this.serverBootstrap = new ServerBootstrap(this.channelFactory);

		ChannelPipeline pipeline = this.serverBootstrap.getPipeline();
		
		pipeline.addLast("encoder", new Encoder());
		pipeline.addLast("decoder", new Decoder());
		pipeline.addLast("handler", new ClientHandler());
		pipeline.addLast("pipelineExecutor", new ExecutionHandler( new OrderedMemoryAwareThreadPoolExecutor(200, 1048576, 1073741824, 100, TimeUnit.MILLISECONDS, Executors.defaultThreadFactory())));
		
		this.serverBootstrap.bind(new InetSocketAddress(ip, port));
	}
}
