package com.sulake.habbo.communication.messages;

import java.util.HashMap;
import java.util.Map;

import com.sulake.habbo.communication.headers.Events;
import com.sulake.habbo.communication.messages.catalog.OpenCatalogMessageEvent;
import com.sulake.habbo.communication.messages.catalog.OpenCatalogPageMessageEvent;
import com.sulake.habbo.communication.messages.catalog.PurchaseCatalogItemMessageEvent;
import com.sulake.habbo.communication.messages.handshake.CheckReleaseMessageEvent;
import com.sulake.habbo.communication.messages.handshake.InitCryptoMessageEvent;
import com.sulake.habbo.communication.messages.handshake.SSOTicketMessageEvent;
import com.sulake.habbo.communication.messages.handshake.SecretKeyMessageEvent;
import com.sulake.habbo.communication.messages.navigator.LoadAllRoomsMessageEvent;
import com.sulake.habbo.communication.messages.navigator.LoadCategorysMessageEvent;
import com.sulake.habbo.communication.messages.navigator.LoadFeaturedRoomsMessageEvent;
import com.sulake.habbo.communication.messages.navigator.LoadMyRoomsMessageEvent;
import com.sulake.habbo.communication.messages.rooms.AddUserToRoomMessageEvent;
import com.sulake.habbo.communication.messages.rooms.LoadFirstRoomDataMessageEvent;
import com.sulake.habbo.communication.messages.rooms.LoadHeightMapMessageEvent;
import com.sulake.habbo.communication.messages.rooms.actions.ApplyActionMessageEvent;
import com.sulake.habbo.communication.messages.rooms.actions.ApplyDanceMessageEvent;
import com.sulake.habbo.communication.messages.rooms.actions.ApplySignMessageEvent;
import com.sulake.habbo.communication.messages.rooms.actions.MoveMessageEvent;
import com.sulake.habbo.communication.messages.rooms.actions.ShoutMessageEvent;
import com.sulake.habbo.communication.messages.rooms.actions.TalkMessageEvent;
import com.sulake.habbo.communication.messages.rooms.items.AddFloorItemMessageEvent;
import com.sulake.habbo.communication.messages.rooms.items.interactions.OpenDiceMessageEvent;
import com.sulake.habbo.communication.messages.types.Event;
import com.sulake.habbo.communication.messages.types.IMessageEvent;
import com.sulake.habbo.communication.messages.users.ChangeLookMessageEvent;
import com.sulake.habbo.communication.messages.users.ChangeMottoMessageEvent;
import com.sulake.habbo.communication.messages.users.GetUserBadgesMessageEvent;
import com.sulake.habbo.communication.messages.users.OpenInventoryMessageEvent;
import com.sulake.habbo.communication.messages.users.UserInformationMessageEvent;
import com.sulake.habbo.utilities.Logger;
import com.sulake.sessions.types.Session;

public class MessageHandler {

	public Map<Short, IMessageEvent> handledMessages;
	public MessageHandler() {
		this.handledMessages = new HashMap<Short, IMessageEvent>();
	}
	
	public void addHandledMessages() {
		
		// Handshake
		getEvents().put(Events.CheckReleaseMessageEvent, new CheckReleaseMessageEvent());
		getEvents().put(Events.InitCryptoMessageEvent, new InitCryptoMessageEvent());
		getEvents().put(Events.SecretKeyMessageEvent, new SecretKeyMessageEvent());
		getEvents().put(Events.SSOTicketMessageEvent, new SSOTicketMessageEvent());
		
		// User
		getEvents().put(Events.UserInformationMessageEvent, new UserInformationMessageEvent());
		getEvents().put(Events.ChangeMottoMessageEvent, new ChangeMottoMessageEvent());
		getEvents().put(Events.ChangeLookMessageEvent, new ChangeLookMessageEvent());
		getEvents().put(Events.GetUserBadgesMessageEvent, new GetUserBadgesMessageEvent());
		getEvents().put(Events.OpenInventoryMessageEvent, new OpenInventoryMessageEvent());
		
		// Navigator
		getEvents().put(Events.LoadCategorysMessageEvent, new LoadCategorysMessageEvent());
		getEvents().put(Events.LoadFeaturedRoomsMessageEvent, new LoadFeaturedRoomsMessageEvent());
		getEvents().put(Events.LoadAllRoomsMessageEvent, new LoadAllRoomsMessageEvent());
		getEvents().put(Events.LoadMyRoomsMessageEvent, new LoadMyRoomsMessageEvent());
		
		// Catalog
		getEvents().put(Events.OpenCatalogMessageEvent, new OpenCatalogMessageEvent());
		getEvents().put(Events.OpenCatalogPageMessageEvent, new OpenCatalogPageMessageEvent());
		getEvents().put(Events.PurchaseCatalogItemMessageEvent, new PurchaseCatalogItemMessageEvent());
		
		// Room
		getEvents().put(Events.LoadFirstRoomDataMessageEvent, new LoadFirstRoomDataMessageEvent());
		getEvents().put(Events.LoadHeightMapMessageEvent, new LoadHeightMapMessageEvent());
		getEvents().put(Events.AddUserToRoomMessageEvent, new AddUserToRoomMessageEvent());
		
		// Room Items
		getEvents().put(Events.AddFloorItemMessageEvent, new AddFloorItemMessageEvent());
		
		// Room Actions
		getEvents().put(Events.TalkMessageEvent, new TalkMessageEvent());
		getEvents().put(Events.ShoutMessageEvent, new ShoutMessageEvent());
		getEvents().put(Events.MoveMessageEvent, new MoveMessageEvent());
		getEvents().put(Events.ApplyDanceMessageEvent, new ApplyDanceMessageEvent());
		getEvents().put(Events.ApplyActionMessageEvent, new ApplyActionMessageEvent());
		getEvents().put(Events.ApplySignMessageEvent, new ApplySignMessageEvent());
		
		// Room Item Interactions
		getEvents().put(Events.OpenDiceMessageEvent, new OpenDiceMessageEvent());
		
		Logger.write("Loaded " + this.getEvents().size() + " message events");
	}
	
	public void handleEvent(Event event, Session session) {
		
		if(getEvents().containsKey(event.getMessageHeader())) {	
			logPacket(event, true);	
			getEvents().get(event.getMessageHeader()).parse(session, event);
		} else {
			if(Events.valueOfId(event.getMessageHeader()) == "") { return; }
			logPacket(event, false);
		}
	}
	
	public void logPacket(Event event, boolean handled) {
		if(handled) {
			Logger.write("[RCV][" + event.getMessageHeader() + "][" + Events.valueOfId(event.getMessageHeader()) + " / Handled] " + event.getMessageBody());
		} else {
			Logger.write("[RCV][" + event.getMessageHeader() + "][" + Events.valueOfId(event.getMessageHeader()) + " / Unhandled] " + event.getMessageBody());
		}
	}
	
	public Map<Short, IMessageEvent> getEvents() {
		return this.handledMessages;
	}
}
