package com.sulake.habbo.communication.messages.catalog;

import java.sql.PreparedStatement;
import java.sql.ResultSet;

import com.sulake.habbo.Habbo;
import com.sulake.habbo.catalog.types.CatalogItem;
import com.sulake.habbo.catalog.types.CatalogPage;
import com.sulake.habbo.communication.headers.Composers;
import com.sulake.habbo.communication.messages.types.Composer;
import com.sulake.habbo.communication.messages.types.Event;
import com.sulake.habbo.communication.messages.types.IMessageEvent;
import com.sulake.sessions.types.Session;

public class PurchaseCatalogItemMessageEvent implements IMessageEvent {

	@Override
	public void parse(Session client, Event event) {
		try {
			int PageId = event.readInt();
			int ItemId = event.readInt();
			
			CatalogPage page = Habbo.getCore().getCatalog().getPage(PageId);
			CatalogItem item = page.getItems().get(ItemId);
			
			if(client.getHabbo().Credits < item.CostCredits)
				return;
			
			if(client.getHabbo().Pixels < item.CostPixels)
				return;
			
			if(client.getHabbo().Diamonds < item.CostOther) 
				return;
			
			if(item.CostCredits > 0)
				client.getHabbo().Credits -= item.CostCredits;
			
			if(item.CostPixels > 0) 
				client.getHabbo().Pixels -= item.CostPixels;
			
			if(item.CostOther > 0) 
				client.getHabbo().Diamonds -= item.CostOther;
			
			client.getHabbo().save();
			
			if(item.CostCredits > 0)
				client.send(client.getHabbo().reloadCreditBalance());
			
			if(item.CostPixels > 0 || item.CostOther > 0)
				client.send(client.getHabbo().reloadPixelBalance());
			
			Composer msg = new Composer(Composers.SerializePurchaseInformationMessageComposer);
			
			msg.appendInt32(item.Id);
			msg.appendString(item.DisplayName);
			msg.appendInt32(item.CostCredits);
			msg.appendInt32(item.CostPixels);
			msg.appendInt32(item.CostOther);
			msg.appendBoolean(false);
			msg.appendInt32(0);
			msg.appendInt32(0);
			msg.appendBoolean(false);
			client.send(msg);
			
			for(int i = 0; i != item.Amount; i++) {
				PreparedStatement stmt = Habbo.getCore().getDatabase().queryParams("INSERT into habbo_items (`item_id`, `item_name`, `owner_id`) VALUES(?, ?, ?)");
				stmt.setInt(1, item.getDefinition().Id);
				stmt.setString(2, item.DisplayName);
				stmt.setInt(3, client.getHabbo().Id);
				stmt.executeUpdate();
				
				ResultSet data = stmt.getGeneratedKeys();
				
				if(data.next()) {
					int Id = data.getInt(1);
					
					client.getHabbo().getInventory().addItem(Id, item.DisplayName, item.ItemId);
				}
			}
			
			Composer purchaseAlert = new Composer(Composers.SendPurchaseAlertMessageComposer);
			purchaseAlert.appendInt32(0);
			
			int i = 2;
			
			if(item.getDefinition().ItemType == "s") {
				// if it's a pet, i = 3..
				i = 1; 
			}
			
			purchaseAlert.appendInt32(i);
			purchaseAlert.appendInt32(item.Amount);
			
			client.send(purchaseAlert);
			
			client.send(new Composer(Composers.UpdateInventoryMessageComposer));			
		
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

}












