package com.sulake.habbo.communication.messages.handshake;

import com.sulake.habbo.communication.headers.Composers;
import com.sulake.habbo.communication.messages.types.Composer;
import com.sulake.habbo.communication.messages.types.Event;
import com.sulake.habbo.communication.messages.types.IMessageEvent;
import com.sulake.habbo.users.dao.UserDao;
import com.sulake.habbo.users.types.User;
import com.sulake.habbo.utilities.InputFilter;
import com.sulake.sessions.types.Session;

public class SSOTicketMessageEvent implements IMessageEvent {

	@Override
	public void parse(Session client, Event event) {
		
		String auth = InputFilter.filterString(event.readString(), false);

		if(!UserDao.ticketExists(auth)) { 
			client.disconnect();
			return;
		}

		User user = UserDao.loadUserByTicket(auth);
		client.setHabbo(user);
		
		client.onLogin();
		
		Composer authOK = new Composer(Composers.AuthenticationOKMessageComposer);
		client.send(authOK);
		
		client.send(client.getHabbo().reloadCreditBalance());
		client.send(client.getHabbo().reloadPixelBalance());
		
		Composer fuserights = new Composer(Composers.FuserightsMessageComposer);
		fuserights.appendInt32(2); // is VIP
		fuserights.appendInt32(client.getHabbo().getRank().Id); // rank
		client.send(fuserights);
		
		if(client.getHabbo().getRank().hasPermission("mod_tool")) {
			Composer modtool = new Composer(Composers.OpenModToolsMessageComposer);
			modtool.appendInt32(0); // user message presets
			modtool.appendInt32(0);
			modtool.appendInt32(0);
			modtool.appendBoolean(true);
			modtool.appendBoolean(true);
			modtool.appendBoolean(true);
			modtool.appendBoolean(true);
			modtool.appendBoolean(true);
			modtool.appendBoolean(true);
			modtool.appendBoolean(true);
			modtool.appendInt32(0); // Room Message presets
			
			client.send(modtool);
		}
	}

}
