package com.sulake.habbo.communication.messages.rooms;

import com.sulake.habbo.Habbo;
import com.sulake.habbo.communication.headers.Composers;
import com.sulake.habbo.communication.messages.types.Composer;
import com.sulake.habbo.communication.messages.types.Event;
import com.sulake.habbo.communication.messages.types.IMessageEvent;
import com.sulake.habbo.flats.items.types.FlatItem;
import com.sulake.habbo.flats.types.FlatInstance;
import com.sulake.habbo.users.dao.UserDao;
import com.sulake.habbo.users.types.User;
import com.sulake.sessions.types.Session;

public class AddUserToRoomMessageEvent implements IMessageEvent {

	@Override
	public void parse(Session client, Event event) {
		try {
			FlatInstance room = Habbo.getCore().getFlats().loadFlat(client.getHabbo().getAvatar().loadingRoom);
			
			User user;
			if(Habbo.getCore().getSessions().getClientByHabbo(room.getData().Owner) != null) {
				user = Habbo.getCore().getSessions().getClientByHabbo(room.getData().Owner).getHabbo();
			} else {
				user = UserDao.loadUserByUsername(room.getData().Owner);
			}
			
			client.getHabbo().getAvatar().loadingRoom = 0;
			
			Composer floorItems = new Composer(Composers.SerializeFloorItemsMessageComposer);
			floorItems.appendInt32(1);
			floorItems.appendInt32(user.Id);
			floorItems.appendString(room.getData().Owner);
			floorItems.appendInt32(room.getItemManager().getItems().size());
			
			for(FlatItem item : room.getItemManager().getItems().values()) {
				if(item.isWallItem == false) {
					item.serialize(floorItems);
					floorItems.appendInt32(user.Id);
				}
			}
			
			client.send(floorItems);
			
			Composer wallItems = new Composer(Composers.SerializeWallItemsMessageComposer);
			wallItems.appendInt32(1);
			wallItems.appendInt32(user.Id);
			wallItems.appendString(room.getData().Owner);
			
			wallItems.appendInt32(0); //room.getItemManager().getItems().size());
			
			//for(FlatItem item : room.getItemManager().getItems().values()) {
			//	if(item.isWallItem) {
			//		item.serialize(wallItems);
			//	}
			//}
			
			client.send(wallItems);
			
			room.getClients().put(client.getHabbo().Id, client);
			client.getHabbo().getAvatar().currentRoom = room;
			client.getHabbo().getAvatar().inRoom = true;
			
			client.getHabbo().getAvatar().X = client.getHabbo().getAvatar().getCurrentFlat().getData().Model.DoorX;
			client.getHabbo().getAvatar().Y = client.getHabbo().getAvatar().getCurrentFlat().getData().Model.DoorY;
			client.getHabbo().getAvatar().Rotation = client.getHabbo().getAvatar().getCurrentFlat().getData().Model.getDoorRot();
			
			Composer users = new Composer(Composers.SetRoomUserMessageComposer);
			users.appendInt32(room.getClients().size());
			
			for(Session c : room.getClients().values()) {
				c.getHabbo().getAvatar().serialize(users);
			}
			
			client.send(users);
			
			Composer states = new Composer(Composers.UpdateStateMessageComposer);
			states.appendInt32(room.getClients().size());
			
			for(Session c : room.getClients().values()) {
				c.getHabbo().getAvatar().serializeStatus(states);			
			}
			
			client.send(states);
		
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
}
