package com.sulake.habbo.communication.remote;

import java.nio.charset.Charset;
import java.util.Map;
import java.util.HashMap;

import org.jboss.netty.buffer.ChannelBuffer;
import org.jboss.netty.buffer.ChannelBuffers;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import com.sulake.habbo.communication.remote.requests.IWebRequest;
import com.sulake.habbo.communication.remote.requests.cache.ReloadCatalog;
import com.sulake.habbo.communication.remote.requests.rooms.GetActiveRooms;
import com.sulake.habbo.communication.remote.requests.users.BroadcastMessage;
import com.sulake.habbo.communication.remote.requests.users.GetUserInfo;

public class RemoteRequestHandler {
	public Map<String, IWebRequest> requests;
	
	public RemoteRequestHandler() {
		this.requests = new HashMap<String, IWebRequest>();
		this.addRequests();
	}
	
	public void addRequests() {
		// User-related Requests
		this.getRequests().put("get_user", new GetUserInfo());
		this.getRequests().put("broadcast_message", new BroadcastMessage());
		
		// Cache-related Requests
		this.getRequests().put("reload_catalog", new ReloadCatalog());
		
		// Housekeeping-related Requests
		this.getRequests().put("get_rooms", new GetActiveRooms());
	}
	
	private Map<String, IWebRequest> getRequests() {
		return requests;
	}
	
	public ChannelBuffer handleRequest(String url) {
		String[] params = url.split("/");
		String key = params[1];
		
		if(this.getRequests().containsKey(key)) {
			try {
				String jsonString = (this.getRequests().get(key).parse(params).get());
				JsonParser parser = new JsonParser();
				
				Gson gson = new GsonBuilder().setPrettyPrinting().create();
				
				JsonElement el = parser.parse(jsonString);
				jsonString = gson.toJson(el); // done
				
				return ChannelBuffers.copiedBuffer(jsonString, Charset.defaultCharset());
			} catch(Exception e) {
				e.printStackTrace();
				return ChannelBuffers.copiedBuffer("Exception caught.", Charset.defaultCharset());
			}
		} else {
			return ChannelBuffers.copiedBuffer("Oops, could not handle request for '" + key + "'", Charset.defaultCharset());
		}
	}
}
