package com.sulake.habbo.flats;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import com.sulake.habbo.Core;
import com.sulake.habbo.database.DatabaseManager;
import com.sulake.habbo.flats.types.FlatData;
import com.sulake.habbo.flats.types.FlatInstance;

public class FlatManager {
	private Map<Integer, FlatData> cachedFlats;
	private Map<Integer, FlatInstance> activeFlats;
	private DatabaseManager db;
	
	public FlatManager(DatabaseManager db) {
		this.db = db;
		this.cachedFlats = new HashMap<Integer, FlatData>();
		this.activeFlats = new HashMap<Integer, FlatInstance>();
	}
	
	public void addFlat(FlatInstance flat) {
		this.getFlats().put(flat.getData().Id, flat);
	}

	public FlatInstance loadFlat(int Id) {
		
		if(this.getFlats().containsKey(Id)) {
			return this.getFlats().get(Id);
		}
		
		FlatInstance f = new FlatInstance(Id);
		
		this.addFlat(f);
		
		return f;
	}
	
	public void unloadFlat(int Id) {
		if(this.getFlats().containsKey(Id)) {
			this.getFlats().get(Id).onUnload();
			this.getFlats().remove(Id);
		}
	}

	public FlatData loadFlatDataById(int id) {
		if(this.getCachedFlats().containsKey(id)) {
			return this.getCachedFlats().get(id);
		} else {
			
			ResultSet row = this.db.readRow("SELECT * FROM `flats` WHERE `id` = " + id + ";");
			FlatData f = null;
			
			f = this.generateFlatData(row);
			this.getCachedFlats().put(f.Id, f);
		
			return f;
		}
	}
	
	public Map<Integer, FlatData> findFlatsByUsername(String username) {
		Map<Integer, FlatData> m = new HashMap<Integer, FlatData>();
		
		for(FlatData f : this.getCachedFlats().values())
		{
			if(f.Owner == username) {
				m.put(f.Id, f);
			}
		}
		
		ResultSet row = this.db.readTable("SELECT * FROM `flats` WHERE `owner` = '" + Core.clean(username) + "';");
		FlatData f = null;
		
		if(row != null) {
			try {
				while(row.next()) {
					f = this.generateFlatData(row);
					m.put(f.Id, f);
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return m;
	}
	
	public FlatData generateFlatData(ResultSet row) {
		FlatData f;
		try {
			f = new FlatData(row.getInt("id"), row.getString("name"), row.getString("description"), row.getString("owner"), row.getString("state"), row.getInt("users"), row.getInt("users_max"), row.getInt("category"), row.getInt("score"), row.getInt("group_id"), row.getString("tags"), row.getString("wallpaper"), row.getString("floor"), row.getString("landscape"), row.getString("model"));
			return f;
		} catch (SQLException e) {
			e.printStackTrace();
		}	
		
		return null;
	}
	
	public Map<Integer, FlatData> getCachedFlats() {
		return this.cachedFlats;
	}
	
	public Map<Integer, FlatInstance> getFlats() {
		return this.activeFlats;
	}
}
