package com.sulake.habbo.flats.types;

import java.util.HashMap;
import java.util.Map;
import com.sulake.habbo.Habbo;
import com.sulake.habbo.communication.headers.Composers;
import com.sulake.habbo.communication.messages.types.Composer;
import com.sulake.habbo.flats.items.ItemManager;
import com.sulake.habbo.flats.rights.RightsManager;
import com.sulake.habbo.users.avatars.types.Avatar;
import com.sulake.sessions.types.Session;

public class FlatInstance {

	public Map<Integer, Session> clients;
	private FlatData Data;
	public RightsManager rights;
	
	private ItemManager items;
	
	public FlatInstance(int id) {
		this.clients = new HashMap<Integer, Session>();
		
		this.Data = Habbo.getCore().getFlats().loadFlatDataById(id);
		
		this.items = new ItemManager(this);
		this.rights = new RightsManager(this);
	}
	
	public void onUnload() {
		
		for(Session client : this.getClients().values()) {
			client.getHabbo().getAvatar().currentRoom = null;
			
			Composer outofroom = new Composer(Composers.OutOfRoomMessageComposer);
			client.send(outofroom);
		}
		
		this.clients.clear();
		
		this.clients = null;
		this.Data = null;
		
		this.getItemManager().items.clear();
		this.getItemManager().items = null;
		
		this.getItemManager().room = null;
		this.items = null;
		
	}
	
	public boolean checkUserCoordinates(int x, int y) {
		for(Session client : this.getClients().values()) {
			if(client.getHabbo().getAvatar().X == x && client.getHabbo().getAvatar().Y == y) {
				return true;
			}
		}
		return false;
	}
	
	public Avatar getAvatarById(int Id) {
		for(Session client : this.getClients().values()) {
			if(client.getHabbo().getAvatar() != null && client.getHabbo().Id == Id) {
				return client.getHabbo().getAvatar();
			}
		}
		
		return null;
	}

	public FlatData getData() {
		return this.Data;
	}
	
	public ItemManager getItemManager() {
		return this.items;
	}
	
	public Map<Integer, Session> getClients() {
		return this.clients;
	}

	public void send(Composer msg) {
		for(Session client : this.getClients().values()) {
			client.send(msg);
		}
	}

	public void removeClient(Session session) {
		this.getClients().remove(session.getHabbo().Id);
	}
	
	public boolean hasRights(int Id) {
		if(this.getData().OwnerId == Id) {
			return true;
		}
		
		if(this.getRights().hasRights(Id)) {
			return true;
		}
		
		return false;
	}
	
	public RightsManager getRights() {
		return this.rights;
	}
}
