package com.sulake.habbo.users.avatars.types;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import com.sulake.habbo.communication.headers.Composers;
import com.sulake.habbo.communication.messages.types.Composer;
import com.sulake.habbo.flats.actions.UserWalker;
import com.sulake.habbo.flats.types.FlatInstance;
import com.sulake.habbo.users.avatars.chat.AvatarChat;
import com.sulake.habbo.users.types.User;

public class Avatar {
	public int loadingRoom;
	public FlatInstance currentRoom;
	
	public User habbo;
	
	public int X;
	public int Y;
	
	public int GoalX;
	public int GoalY;
	
	public int Rotation;
	
	public double Height;
	public boolean selectedSquare;
	
	public UserWalker walker;
	public Thread walkerThread;
	public boolean isDancing = false;
	public boolean isWalking;
	
	public boolean inRoom = false;
	
	public AvatarChat chat;
	
	public Map<String, String> statuses;
	public boolean isSitting;
	
	public Avatar(User habbo) {
		this.habbo = habbo;
		this.Height = 0;
		this.loadingRoom = 0;
		this.currentRoom = null;
		this.chat = new AvatarChat();
		this.statuses = new HashMap<String, String>();
		this.isSitting = false;
	}
	
	
	public FlatInstance getCurrentFlat() {
		return this.currentRoom;
	}
	
	public void sendStatus() {
		Composer msg = new Composer(Composers.UpdateStateMessageComposer);
		msg.appendInt32(1);
		msg.appendInt32(this.getHabbo().Id);
		msg.appendInt32(X);
		msg.appendInt32(Y);
		msg.appendString(this.Height);
		msg.appendInt32(Rotation);
		msg.appendInt32(Rotation);
		
		msg.appendString(getFullStatus());
		
		if(this.inRoom) {
			this.getCurrentFlat().send(msg);
		}
	}
	
	public void serialize(Composer msg) {
		msg.appendInt32(this.getHabbo().Id);
		msg.appendString(this.getHabbo().Username);
		msg.appendString(this.getHabbo().Motto);
		msg.appendString(this.getHabbo().Figure);
		msg.appendInt32(this.getHabbo().Id); // Entity Id
		msg.appendInt32(this.X);
		msg.appendInt32(this.Y);
		msg.appendString(Double.toString(this.Height)); 
		msg.appendInt32(this.Rotation);
		msg.appendInt32(1);
		msg.appendString(this.getHabbo().Gender.toLowerCase()); 
		msg.appendInt32(-1);
		msg.appendInt32(-1);
		msg.appendInt32(0);
		msg.appendInt32(this.getHabbo().achievementScore);
	}
	
	public void serializeStatus(Composer msg) {
		msg.appendInt32(this.getHabbo().Id);
		msg.appendInt32(this.X);
		msg.appendInt32(this.Y);
		msg.appendString(Double.toString(this.Height));
		msg.appendInt32(this.Rotation);
		msg.appendInt32(this.Rotation);
		
		this.addStatus("flatctrl", Integer.toString(4));
		msg.appendString(this.getFullStatus());
	}
	
	public String getFullStatus() {
		String status = "/";

		for (Entry<String, String> set : getStatuses().entrySet()) {
			status += set.getKey() + " " + set.getValue() + "/";
		}

		return status + "/";
	}
	
	public User getHabbo() {
		return this.habbo;
	}

	public void update() {
		Composer msg = new Composer(Composers.UpdateUserInformationMessageComposer);
		msg.appendInt32(this.getHabbo().Id);
		msg.appendString(this.getHabbo().getFigure());
		msg.appendString(this.getHabbo().getGender().toLowerCase());
		msg.appendString(this.getHabbo().getMotto());
		msg.appendInt32(this.getHabbo().achievementScore);
		
		this.getCurrentFlat().send(msg);
		
		Composer otherMsg = new Composer(Composers.UpdateUserInformationMessageComposer);
		otherMsg.appendInt32(-1);
		otherMsg.appendString(this.getHabbo().getFigure());
		otherMsg.appendString(this.getHabbo().getGender().toLowerCase());
		otherMsg.appendString(this.getHabbo().getMotto());
		otherMsg.appendInt32(this.getHabbo().achievementScore);
		
		this.getCurrentFlat().getClients().get(this.getHabbo().Id).send(otherMsg);
		
	}

	public void say(String message, boolean shout, int colour) {
		
		Composer msg;
		
		if(shout) {
			msg = new Composer(Composers.ShoutMessageComposer);
		} else {
			msg = new Composer(Composers.TalkMessageComposer);
		}
		
		msg.appendInt32(this.getHabbo().Id);
		msg.appendString(message);
		msg.appendInt32(this.getChat().getEmotion(message)); // emotion
		msg.appendInt32(colour); // colour
		msg.appendInt32(0);
		msg.appendInt32(-1);
		
		this.getCurrentFlat().send(msg);
	}
	
	public AvatarChat getChat() {
		return this.chat;
	}

	public void addStatus(String key, String value) {
		this.getStatuses().put(key, value);
	}


	public Map<String, String> getStatuses() {
		return this.statuses;
	}


	public void dispose() {
		this.statuses.clear();
		this.statuses = null;
		
		this.getChat().emotions.clear();
		this.chat = null;
		
		this.currentRoom = null;
	}
}
