package com.sulake.habbo.users.types;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Map;

import com.sulake.habbo.Habbo;
import com.sulake.habbo.communication.headers.Composers;
import com.sulake.habbo.communication.messages.types.Composer;
import com.sulake.habbo.flats.types.FlatData;
import com.sulake.habbo.users.avatars.types.Avatar;
import com.sulake.habbo.users.components.BadgeComponent;
import com.sulake.habbo.users.components.BalanceComponent;
import com.sulake.habbo.users.components.InventoryComponent;
import com.sulake.habbo.users.ranks.types.Rank;

public class User {

	public int Id;
	public String Username;
	public String Figure;
	
	public int Credits;
	public int Diamonds;
	public int Pixels;
	
	public String Motto;
	public String Gender;
	
	public Avatar avatar;
	
	public Rank rank;
	
	public Map<Integer, FlatData> flats;
	
	public BalanceComponent balanceComponent;
	public BadgeComponent badgeComponent;
	public InventoryComponent inventoryComponent;
	
	public int achievementScore;
	
	public User(int id, String username, String figure, String motto, String gender, int rankId, int credits, int diamonds, int pixels) {
		this.Id = id;
		this.Username = username;
		this.Figure = figure;
		this.Motto = motto;
		this.Gender = gender;
		
		this.balanceComponent = new BalanceComponent(this.getId());
		this.badgeComponent = new BadgeComponent(this.getId());
		this.inventoryComponent = new InventoryComponent(this.getId());
		
		this.avatar = new Avatar(this);
		
		this.achievementScore = 0; // TODO: Achievement Score
		
		this.rank = Habbo.getCore().getRanks().getRank(rankId);
		this.Credits = credits;
		this.Diamonds = diamonds;
		this.Pixels = pixels;
	}
	
	public void save() {
		try {
			PreparedStatement statement = Habbo.getCore().getDatabase().queryParams("UPDATE habbos SET motto = ?, gender = ?, rank = ?, credits = ?, pixels = ?, vip_points = ?, figure = ? WHERE id = ?");
			statement.setString(1, Motto);
			statement.setString(2, Gender);
			statement.setInt(3, getRank().Id);
			statement.setInt(4, Credits);
			statement.setInt(5, Pixels);
			statement.setInt(6, Diamonds);
			statement.setString(7, Figure);
			statement.setInt(8, Id);

			statement.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	public String getUsername() {
		return this.Username;
	}

	public String getFigure() {
		return this.Figure;
	}
	
	public String getMotto() {
		return this.Motto;
	}
	
	public String getGender() {
		return this.Gender;
	}
	
	public BalanceComponent getBalanceComponent() {
		return this.balanceComponent;
	}
	
	public BadgeComponent getBadgeComponent() {
		return this.badgeComponent;
	}
	
	public InventoryComponent getInventory() {
		return this.inventoryComponent;
	}
	
	public Avatar getAvatar() {
		return this.avatar;
	}
	
	public Rank getRank() {
		return this.rank;
	}
	
	public int getCredits() {
		return this.Credits;
	}

	public int getId() {
		return this.Id;
	}

	public void assignOwnRooms() {
		this.flats = Habbo.getCore().getFlats().findFlatsByUsername(getUsername());
	}

	public Map<Integer, FlatData> getFlats() {
		return this.flats;
	}

	public Composer reloadCreditBalance() {
		Composer msg = new Composer(Composers.CreditsBalanceMessageComposer);
		msg.appendString(this.getCredits() + ".0");
		
		return msg;
	}
	
	public Composer reloadPixelBalance() {
		Composer msg = new Composer(Composers.ActivityPointsMessageComposer);
		msg.appendInt32(2);
		msg.appendInt32(0);
		msg.appendInt32(this.Pixels);
		msg.appendInt32(105);
		msg.appendInt32(this.Diamonds);
		
		return msg;
	}
}
