package com.sulake.sessions.types;

import org.jboss.netty.channel.Channel;

import com.sulake.habbo.Habbo;
import com.sulake.habbo.communication.headers.Composers;
import com.sulake.habbo.communication.messages.types.Composer;
import com.sulake.habbo.settings.ServerSettings;
import com.sulake.habbo.users.dao.UserDao;
import com.sulake.habbo.users.types.User;

public class Session {

	private Channel channel;
	private User habbo;
	
	public Session(Channel channel) {
		this.channel = channel;
	}
	
	public void disconnect() {
		this.getChannel().disconnect();
	}

	public void sendNotif(String string) {
		
		Composer alert = new Composer(Composers.SendNotifMessageComposer);
		alert.appendString(string);
		alert.appendString("");
		this.getChannel().write(alert);
	}
	
	
	public void sendBroadcastMessage(String string) {
        Composer alert = new Composer(Composers.BroadcastMessageMessageComposer);
        alert.appendString(string);
        alert.appendString("");
        this.getChannel().write(alert);
}
	
	public void send(Composer msg) {
		try {
		this.getChannel().write(msg);
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	public void onLogin() {
		
		UserDao.updateUserStatus(true, this.getHabbo().getId());
		
		String welcomeMessage = ServerSettings.get("login_message").value;
		String msg = welcomeMessage.replace("{username}", this.getHabbo().Username);
		this.sendNotif(msg);
		
		try {
			this.getHabbo().assignOwnRooms();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public void onLogout() {
		if(getHabbo().getAvatar().inRoom) {
			if(getHabbo().getAvatar().getCurrentFlat().getClients().size() == 1) {
				Habbo.getCore().getFlats().unloadFlat(getHabbo().getAvatar().getCurrentFlat().getData().Id);
			} else {
				getHabbo().getAvatar().getCurrentFlat().removeClient(this);
			}
		}
		
		getHabbo().getInventory().dispose();
		getHabbo().getAvatar().dispose();
		
		getHabbo().save();
		UserDao.updateUserStatus(false, this.getHabbo().getId());
	}
	
	public Channel getChannel() {
		return this.channel;
	}

	public User getHabbo() {
		return habbo;
	}

	public void setHabbo(User habbo) {
		this.habbo = habbo;
	}
}